@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-/* Copyright (C) 2007-2019  B.A.T.M.A.N. contributors:
+/* Copyright (C) 2007-2020  B.A.T.M.A.N. contributors:
  *
  * Marek Lindner, Simon Wunderlich
  */
@@ -151,25 +151,22 @@ static bool batadv_is_on_batman_iface(const struct net_device *net_dev)
 	struct net *net = dev_net(net_dev);
 	struct net_device *parent_dev;
 	struct net *parent_net;
-	int iflink;
 	bool ret;
 
 	/* check if this is a batman-adv mesh interface */
 	if (batadv_softif_is_valid(net_dev))
 		return true;
 
-	iflink = dev_get_iflink(net_dev);
-	if (iflink == 0)
+	/* no more parents..stop recursion */
+	if (dev_get_iflink(net_dev) == 0 ||
+	    dev_get_iflink(net_dev) == net_dev->ifindex)
 		return false;
 
 	parent_net = batadv_getlink_net(net_dev, net);
 
-	/* iflink to itself, most likely physical device */
-	if (net == parent_net && iflink == net_dev->ifindex)
-		return false;
-
 	/* recurse over the parent device */
-	parent_dev = __dev_get_by_index((struct net *)parent_net, iflink);
+	parent_dev = __dev_get_by_index((struct net *)parent_net,
+					dev_get_iflink(net_dev));
 	/* if we got a NULL parent_dev there is something broken.. */
 	if (!parent_dev) {
 		pr_err("Cannot find parent device\n");
@@ -219,15 +216,14 @@ static struct net_device *batadv_get_real_netdevice(struct net_device *netdev)
 	struct net_device *real_netdev = NULL;
 	struct net *real_net;
 	struct net *net;
-	int iflink;
+	int ifindex;
 
 	ASSERT_RTNL();
 
 	if (!netdev)
 		return NULL;
 
-	iflink = dev_get_iflink(netdev);
-	if (iflink == 0) {
+	if (netdev->ifindex == dev_get_iflink(netdev)) {
 		dev_hold(netdev);
 		return netdev;
 	}
@@ -237,16 +233,9 @@ static struct net_device *batadv_get_real_netdevice(struct net_device *netdev)
 		goto out;
 
 	net = dev_net(hard_iface->soft_iface);
+	ifindex = dev_get_iflink(netdev);
 	real_net = batadv_getlink_net(netdev, net);
-
-	/* iflink to itself, most likely physical device */
-	if (net == real_net && netdev->ifindex == iflink) {
-		real_netdev = netdev;
-		dev_hold(real_netdev);
-		goto out;
-	}
-
-	real_netdev = dev_get_by_index(real_net, iflink);
+	real_netdev = dev_get_by_index(real_net, ifindex);
 
 out:
 	if (hard_iface)
@@ -484,8 +473,8 @@ static void batadv_primary_if_select(struct batadv_priv *bat_priv,
 	if (new_hard_iface)
 		kref_get(&new_hard_iface->refcount);
 
-	curr_hard_iface = rcu_dereference_protected(bat_priv->primary_if, 1);
-	rcu_assign_pointer(bat_priv->primary_if, new_hard_iface);
+	curr_hard_iface = rcu_replace_pointer(bat_priv->primary_if,
+					      new_hard_iface, 1);
 
 	if (!new_hard_iface)
 		goto out;
@@ -565,9 +554,6 @@ static void batadv_hardif_recalc_extra_skbroom(struct net_device *soft_iface)
 	needed_headroom = lower_headroom + (lower_header_len - ETH_HLEN);
 	needed_headroom += batadv_max_header_len();
 
-	/* fragmentation headers don't strip the unicast/... header */
-	needed_headroom += sizeof(struct batadv_frag_packet);
-
 	soft_iface->needed_headroom = needed_headroom;
 	soft_iface->needed_tailroom = lower_tailroom;
 }
